/*
 * engdic.c
 *
 * English dictionary for korean
 *
 * Author: Oh Junseon <hollywar@mail.holywar.net>
 * Date  : $Date: 1999/06/03 03:23:22 $ 
 *
 * $Revision: 1.4 $
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
*/


/* SCSS/RCS Id */
#ifdef LINT
static char rcsid[]="@(#)$Id: engdic.c,v 1.4 1999/06/03 03:23:22 hollywar Exp $";
#endif /* lint */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <zlib.h>  // gzip library //

#include "engdic.h"


void error_exit(char *message) ;
void show_result(char *tk , char *tk2) ;
char *lower_string (char *data) ;
int han_check (char *data) ;


int main (int argc , char *argv[]) 
{
    char buf[COLS_MAX] ;
    char command[MAX_LENGTH]  ;
    char *key_word ;
    char *small_key_word ;
    char *tk  , *small_tk;
    char sep[] = ":" ;
    char first_char ;
    char check=0 ;
    
    // type of gzip file //
    gzFile gzfp ;
    

    if (argc > 1) {
	key_word = argv[1] ;
    } 
	
    // start loop //
    while(1) {
	if (argc <= 1) {
	    // get search query //
	    strcpy(command,"");
	    printf("%s",SEARCH_WORD) ;
	    fgets(command,MAX_LENGTH,stdin) ;

	    // remove enter value //
	    command[strlen(command)-1] = '\0' ;
	    key_word = command ;
	}
	// check null line or CTRL + D for terminate program //
	if (!strcmp(key_word,"") || key_word[0] == 4) {
	    printf("%s\r\n",END_WORD) ;
	    break ;
	}
	
	if (han_check(key_word)) {
	    printf(ERROR_COLOR "%s\r\n" NO_COLOR,HAN_ERROR_WORD) ;
	    if (argc > 1) break ; else continue ;
	}

        // get first keyword character for reading file //
        first_char = tolower(key_word[0]) ;
        sprintf(buf,"%s/%c.dic.gz",INSTALL_DIC_DATA_DIR,first_char) ;
    
	// open gzip file //
        // if ( !(gzfp = gzopen(buf,"rb"))) error_exit("File open failed") ;
        if ( !(gzfp = gzopen(buf,"rb"))) continue ;
    
	
	// Searching keyword //
	check=0 ;
        while( (gzgets(gzfp,buf,COLS_MAX))) {
	    tk = strtok(buf,sep) ;
	    small_tk = lower_string(tk) ;
            small_key_word = lower_string(key_word) ;

	    // Data have one space (remove one space) //
	    small_tk[strlen(small_tk)-1] = '\0' ;
	
	    if (!strcmp(small_tk,small_key_word)) {
		check=1;
		show_result(tk,strtok(NULL,"\0")) ;
	    }
	    if (check && small_tk[1] > small_key_word[1]) {
	        free(small_tk) ;
	        free(small_key_word) ;
	        break ;
	    }
	    free(small_tk) ;
	    free(small_key_word) ;
        }

	// close gzip file //
        gzclose(gzfp) ;
	
	printf("\r\n");
	
	if (argc > 1) break ;
    }

    return 0 ;
}


void show_result(char *tk , char *tk2) 
{
    char *i, *j ;
    unsigned char *data ;
    int check_cols_count = 0 ;
    
    if ( !(data = (char*) malloc(sizeof(char)*80))) error_exit("Can't allocate memory"); 
    
    j = data ;

    printf(WORD_COLOR "[ %s]\r\n" NO_COLOR,tk ) ;
    
    for (i=++tk2 ; *i != '\0' ; i++ ) {
	check_cols_count ++ ;
	if ( (check_cols_count > 65 && *i == ' ') || (check_cols_count > 79) ) {
	    *j = '\0' ;
	    printf(" %s\r\n",data) ; 
	    memset(data,0,80) ;
	    j = data ;
	    check_cols_count=0 ;
	    continue ;
	}
	*j++ = *i ;
    }
    if (check_cols_count != 0) {
	*j = '\0' ;
	printf(" %s",data) ; 
    }
    
    free(data) ;
    
}

char *lower_string (char *data)
{
   char *i, *j ;
   char *code ;

   code = (char *)malloc ((strlen(data)+1) * sizeof (char) );

   for (j=code, i=data; *i!='\0'; i++, j++) {
      if (*i != ' ')
         *j = tolower(*i);
      else *j = ' ' ;
   }
   *j = '\0';

   return code ;
}

int han_check (char *data)
{
    int i=0 ;
	
    for (i=0 ; data[i] != '\0' ; i++) {
        if (data[i] & 0x80) return 1 ;
    }
    return 0 ;
}



void error_exit (char *message)
{
    printf("%s\r\n",message) ;
    exit(1) ;
}


/*
 | $Id: engdic.c,v 1.4 1999/06/03 03:23:22 hollywar Exp $
 |
 | Local Variables:
 | mode: c
 | mode: font-lock
 | version-control: t
 | delete-old-versions: t
 | End:
 |
 | -*- End-Of-File -*-
 */
